"use strict";
this.haketilo_exports = this.haketilo_exports || {};
this.haketilo_exports["background/CORS_bypass_server.js"] = {};

(function() {
var globalThis = this.haketilo_this
{
/**
 * This file is part of Haketilo.
 *
 * Function: Allow other parts of the extension to bypass CORS by routing their
 *           requests through this background script using one-off messages.
 *
 * Copyright (C) 2022 Wojtek Kosior <koszko@koszko.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * As additional permission under GNU GPL version 3 section 7, you
 * may distribute forms of that code without the copy of the GNU
 * GPL normally required by section 4, provided you include this
 * license notice and, in case of non-source distribution, a URL
 * through which recipients can access the Corresponding Source.
 * If you modify file(s) with this exception, you may extend this
 * exception to your version of the file(s), but you are not
 * obligated to do so. If you do not wish to do so, delete this
 * exception statement from your version.
 *
 * As a special exception to the GPL, any HTML file which merely
 * makes function calls to this code, and for that purpose
 * includes it by reference shall be deemed a separate work for
 * copyright law purposes. If you modify this code, you may extend
 * this exception to your version of the code, but you are not
 * obligated to do so. If you do not wish to do so, delete this
 * exception statement from your version.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 * I, Wojtek Kosior, thereby promise not to sue for violation of this file's
 * license. Although I request that you do not make use of this code in a
 * proprietary program, I am not going to enforce this in court.
 */

const browser = haketilo_exports["common/browser.js"].browser;
const uint8_to_hex = haketilo_exports["common/misc.js"].uint8_to_hex;
const error_data_jsonifiable = haketilo_exports["common/misc.js"].error_data_jsonifiable;

/*
 * In this file we implement a fetch()-from-background-script service. Code in
 * other parts of the extension shall call sendMessage() with arguments to
 * fetch() and code here will call fetch() with those arguments and send back
 * the response.
 *
 * We have to convert the Response from fetch() into a JSON-ifiable value and
 * then convert it back (using Response() constructor) due (among others) the
 * limitations of Chromium's messaging API:
 * https://developer.mozilla.org/en-US/docs/Mozilla/Add-ons/WebExtensions/Chrome_incompatibilities#data_cloning_algorithm
 *
 * We also catch possible errors from fetch() (e.g. in case of an invalid URL)
 * and send their data in JSON-ifiable form so that the error object can be
 * later re-created.
 */

/* Make it possible to serialize Response object. */
async function response_data_jsonifiable(response) {
    return {
	status:     response.status,
	statusText: response.statusText,
	headers:    [...response.headers.entries()],
	body:       uint8_to_hex(new Uint8Array(await response.arrayBuffer()))
    };
}

async function perform_download(fetch_data) {
    try {
	const response = await fetch(fetch_data.url, fetch_data.init);
	return response_data_jsonifiable(response);
    } catch(e) {
	return {error: error_data_jsonifiable(e)};
    }
}

function on_CORS_bypass_request([type, fetch_data], sender, respond_cb) {
    if (type !== "CORS_bypass")
	return;

    perform_download(fetch_data).then(respond_cb);

    return true;
}

function start() {
    browser.runtime.onMessage.addListener(on_CORS_bypass_request);
}
this.haketilo_exports["background/CORS_bypass_server.js"].start = (start);
}
}).call({
    haketilo_exports: this.haketilo_exports,
    haketilo_this:    this
});
