"use strict";
this.haketilo_exports = this.haketilo_exports || {};
this.haketilo_exports["background/patterns_query_manager.js"] = {};

(function() {
var globalThis = this.haketilo_this
{
/**
 * This file is part of Haketilo.
 *
 * Function: Instantiate the Pattern Tree data structure, filled with mappings
 *           from IndexedDB.
 *
 * Copyright (C) 2021, 2022 Wojtek Kosior
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * As additional permission under GNU GPL version 3 section 7, you
 * may distribute forms of that code without the copy of the GNU
 * GPL normally required by section 4, provided you include this
 * license notice and, in case of non-source distribution, a URL
 * through which recipients can access the Corresponding Source.
 * If you modify file(s) with this exception, you may extend this
 * exception to your version of the file(s), but you are not
 * obligated to do so. If you do not wish to do so, delete this
 * exception statement from your version.
 *
 * As a special exception to the GPL, any HTML file which merely
 * makes function calls to this code, and for that purpose
 * includes it by reference shall be deemed a separate work for
 * copyright law purposes. If you modify this code, you may extend
 * this exception to your version of the code, but you are not
 * obligated to do so. If you do not wish to do so, delete this
 * exception statement from your version.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 * I, Wojtek Kosior, thereby promise not to sue for violation of this file's
 * license. Although I request that you do not make use of this code in a
 * proprietary program, I am not going to enforce this in court.
 */

const pqt = haketilo_exports["common/patterns_query_tree.js"];
const haketilodb = haketilo_exports["common/indexeddb.js"];

const browser = haketilo_exports["common/browser.js"].browser;

let default_allow = {};
this.haketilo_exports["background/patterns_query_manager.js"].default_allow = (default_allow);

let secret;

const tree = pqt.make();
this.haketilo_exports["background/patterns_query_manager.js"].tree = (tree);

const currently_registered = new Map();

let registered_script = null;
let script_update_occuring = false;
let script_update_needed;

async function update_content_script() {
    if (script_update_occuring)
	return;

    script_update_occuring = true;

    while (script_update_needed) {
	script_update_needed = false;

	const code = `\
this.haketilo_secret        = ${JSON.stringify(secret)};
this.haketilo_pattern_tree  = ${JSON.stringify(tree)};
this.haketilo_default_allow = ${JSON.stringify(default_allow.value)};
if (this.haketilo_content_script_main)
    haketilo_content_script_main();`;

	const new_script = await browser.contentScripts.register({
	    "js": [{code}],
	    "matches": ["<all_urls>"],
	    "allFrames": true,
	    "runAt": "document_start"
	});

	if (registered_script)
	    registered_script.unregister();

	registered_script = new_script;
    }

    script_update_occuring = false;
}

function register(kind, object) {
    if (kind === "mappings") {
	for (const [pattern, resource] of Object.entries(object.payloads || {}))
	    pqt.register(tree, pattern, object.identifier, resource);
    } else /* if (kind === "blocking") */ {
	/*
	 * All simple block/allow rules use "~allow" in place of mapping id.
	 * This way it won't collide with any real mapping id and will always
	 * be sorted as higher value than mapping ids.
	 */
	pqt.register(tree, object.pattern, "~allow", object.allow + 0);
    }

    const id = kind === "mappings" ? object.identifier : object.pattern;
    currently_registered.set(id, object);
}

function changed(kind, change) {
    const old_version = currently_registered.get(change.key);
    if (old_version !== undefined) {
	if (kind === "mappings") {
	    for (const pattern in old_version.payloads || {})
		pqt.deregister(tree, pattern, change.key);
	} else /* if (kind === "blocking") */ {
	    pqt.deregister(tree, change.key, "~allow");
	}

	currently_registered.delete(change.key);
    }

    if (change.new_val !== undefined)
	register(kind, change.new_val);

    script_update_needed = true;
    setTimeout(update_content_script, 0);
}

function setting_changed(change) {
    if (change.key !== "default_allow")
	return;

    default_allow.value = (change.new_val || {}).value;

    script_update_needed = true;
    setTimeout(update_content_script, 0);
}

async function start(secret_) {
    secret = secret_;

    const [mapping_tracking, initial_mappings] =
	  await haketilodb.track.mapping(ch => changed("mappings", ch));
    const [blocking_tracking, initial_blocking] =
	  await haketilodb.track.blocking(ch => changed("blocking", ch));

    initial_mappings.forEach(m => register("mappings", m));
    initial_blocking.forEach(b => register("blocking", b));

    const [setting_tracking, initial_settings] =
	  await haketilodb.track.setting(setting_changed);

    for (const setting of initial_settings) {
	if (setting.name === "default_allow")
	    Object.assign(default_allow, setting);
    }

    script_update_needed = true;
    await update_content_script();
}
this.haketilo_exports["background/patterns_query_manager.js"].start = (start);
}
}).call({
    haketilo_exports: this.haketilo_exports,
    haketilo_this:    this
});
