"use strict";
this.haketilo_exports = this.haketilo_exports || {};
this.haketilo_exports["common/indexeddb.js"] = {};

(function() {
var globalThis = this.haketilo_this
{
/**
 * This file is part of Haketilo.
 *
 * Function: Facilitate use of IndexedDB within Haketilo.
 *
 * Copyright (C) 2021, 2022 Wojtek Kosior <koszko@koszko.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * As additional permission under GNU GPL version 3 section 7, you
 * may distribute forms of that code without the copy of the GNU
 * GPL normally required by section 4, provided you include this
 * license notice and, in case of non-source distribution, a URL
 * through which recipients can access the Corresponding Source.
 * If you modify file(s) with this exception, you may extend this
 * exception to your version of the file(s), but you are not
 * obligated to do so. If you do not wish to do so, delete this
 * exception statement from your version.
 *
 * As a special exception to the GPL, any HTML file which merely
 * makes function calls to this code, and for that purpose
 * includes it by reference shall be deemed a separate work for
 * copyright law purposes. If you modify this code, you may extend
 * this exception to your version of the code, but you are not
 * obligated to do so. If you do not wish to do so, delete this
 * exception statement from your version.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 * I, Wojtek Kosior, thereby promise not to sue for violation of this file's
 * license. Although I request that you do not make use of this code in a
 * proprietary program, I am not going to enforce this in court.
 */

const entities = haketilo_exports["common/entities.js"];
const broadcast = haketilo_exports["common/broadcast.js"];

let initial_data = (
{
    "resource": {
	"haketilo-demo-script": {
	    "2022.1.28": {
		"$schema": "https://hydrilla.koszko.org/schemas/api_resource_description-1.schema.json",
		"source_name": "haketilo-default-settings",
		"source_copyright": [{
		    "file": "CC0-1.0.txt",
		    "sha256": "a2010f343487d3f7618affe54f789f5487602331c0a8d03f49e9a7c547cf0499"
		}],
		"type": "resource",
		"identifier": "haketilo-demo-script",
		"long_name": "Haketilo demonstrational script",
		"uuid": "9dccba55-4b18-4a01-933f-feaabd62fe95",
		"version": [2022, 1, 28],
		"revision": 1,
		"description": "Demonstrational script that displays a message with green background on given page.",
		"dependencies": [],
		"scripts": [{
		    "file": "demo-script.js",
		    "sha256": "f1d9dc9f2f4edaaebd9167eb8b79a6da6434313df47921aa4ea8ae278f7739e3"
		}]
	    }
	}
    },
    "mapping": {
	"haketilo-demo-message": {
	    "2022.1.28": {
		"$schema": "https://hydrilla.koszko.org/schemas/api_mapping_description-1.schema.json",
		"source_name": "haketilo-default-settings",
		"source_copyright": [{
		    "file": "CC0-1.0.txt",
		    "sha256": "a2010f343487d3f7618affe54f789f5487602331c0a8d03f49e9a7c547cf0499"
		}],
		"type": "mapping",
		"identifier": "haketilo-demo-message",
		"long_name": "Haketilo demonstrational message",
		"uuid": "2caf5000-cc22-40e4-ba3e-e263f196a1ac",
		"version": [2022, 1, 28],
		"description": "Demonstrate Haketilo's capabilities by displaying a simple message on hachette-hydrilla.org",
		"payloads": {
		    "https://hachette-hydrilla.org": {
			"identifier": "haketilo-demo-script"
		    },
		}
	    }
	}
    },
    "file": {
	"sha256": {
	    "a2010f343487d3f7618affe54f789f5487602331c0a8d03f49e9a7c547cf0499": "Creative Commons Legal Code\n\nCC0 1.0 Universal\n\n    CREATIVE COMMONS CORPORATION IS NOT A LAW FIRM AND DOES NOT PROVIDE\n    LEGAL SERVICES. DISTRIBUTION OF THIS DOCUMENT DOES NOT CREATE AN\n    ATTORNEY-CLIENT RELATIONSHIP. CREATIVE COMMONS PROVIDES THIS\n    INFORMATION ON AN \"AS-IS\" BASIS. CREATIVE COMMONS MAKES NO WARRANTIES\n    REGARDING THE USE OF THIS DOCUMENT OR THE INFORMATION OR WORKS\n    PROVIDED HEREUNDER, AND DISCLAIMS LIABILITY FOR DAMAGES RESULTING FROM\n    THE USE OF THIS DOCUMENT OR THE INFORMATION OR WORKS PROVIDED\n    HEREUNDER.\n\nStatement of Purpose\n\nThe laws of most jurisdictions throughout the world automatically confer\nexclusive Copyright and Related Rights (defined below) upon the creator\nand subsequent owner(s) (each and all, an \"owner\") of an original work of\nauthorship and/or a database (each, a \"Work\").\n\nCertain owners wish to permanently relinquish those rights to a Work for\nthe purpose of contributing to a commons of creative, cultural and\nscientific works (\"Commons\") that the public can reliably and without fear\nof later claims of infringement build upon, modify, incorporate in other\nworks, reuse and redistribute as freely as possible in any form whatsoever\nand for any purposes, including without limitation commercial purposes.\nThese owners may contribute to the Commons to promote the ideal of a free\nculture and the further production of creative, cultural and scientific\nworks, or to gain reputation or greater distribution for their Work in\npart through the use and efforts of others.\n\nFor these and/or other purposes and motivations, and without any\nexpectation of additional consideration or compensation, the person\nassociating CC0 with a Work (the \"Affirmer\"), to the extent that he or she\nis an owner of Copyright and Related Rights in the Work, voluntarily\nelects to apply CC0 to the Work and publicly distribute the Work under its\nterms, with knowledge of his or her Copyright and Related Rights in the\nWork and the meaning and intended legal effect of CC0 on those rights.\n\n1. Copyright and Related Rights. A Work made available under CC0 may be\nprotected by copyright and related or neighboring rights (\"Copyright and\nRelated Rights\"). Copyright and Related Rights include, but are not\nlimited to, the following:\n\n  i. the right to reproduce, adapt, distribute, perform, display,\n     communicate, and translate a Work;\n ii. moral rights retained by the original author(s) and/or performer(s);\niii. publicity and privacy rights pertaining to a person's image or\n     likeness depicted in a Work;\n iv. rights protecting against unfair competition in regards to a Work,\n     subject to the limitations in paragraph 4(a), below;\n  v. rights protecting the extraction, dissemination, use and reuse of data\n     in a Work;\n vi. database rights (such as those arising under Directive 96/9/EC of the\n     European Parliament and of the Council of 11 March 1996 on the legal\n     protection of databases, and under any national implementation\n     thereof, including any amended or successor version of such\n     directive); and\nvii. other similar, equivalent or corresponding rights throughout the\n     world based on applicable law or treaty, and any national\n     implementations thereof.\n\n2. Waiver. To the greatest extent permitted by, but not in contravention\nof, applicable law, Affirmer hereby overtly, fully, permanently,\nirrevocably and unconditionally waives, abandons, and surrenders all of\nAffirmer's Copyright and Related Rights and associated claims and causes\nof action, whether now known or unknown (including existing as well as\nfuture claims and causes of action), in the Work (i) in all territories\nworldwide, (ii) for the maximum duration provided by applicable law or\ntreaty (including future time extensions), (iii) in any current or future\nmedium and for any number of copies, and (iv) for any purpose whatsoever,\nincluding without limitation commercial, advertising or promotional\npurposes (the \"Waiver\"). Affirmer makes the Waiver for the benefit of each\nmember of the public at large and to the detriment of Affirmer's heirs and\nsuccessors, fully intending that such Waiver shall not be subject to\nrevocation, rescission, cancellation, termination, or any other legal or\nequitable action to disrupt the quiet enjoyment of the Work by the public\nas contemplated by Affirmer's express Statement of Purpose.\n\n3. Public License Fallback. Should any part of the Waiver for any reason\nbe judged legally invalid or ineffective under applicable law, then the\nWaiver shall be preserved to the maximum extent permitted taking into\naccount Affirmer's express Statement of Purpose. In addition, to the\nextent the Waiver is so judged Affirmer hereby grants to each affected\nperson a royalty-free, non transferable, non sublicensable, non exclusive,\nirrevocable and unconditional license to exercise Affirmer's Copyright and\nRelated Rights in the Work (i) in all territories worldwide, (ii) for the\nmaximum duration provided by applicable law or treaty (including future\ntime extensions), (iii) in any current or future medium and for any number\nof copies, and (iv) for any purpose whatsoever, including without\nlimitation commercial, advertising or promotional purposes (the\n\"License\"). The License shall be deemed effective as of the date CC0 was\napplied by Affirmer to the Work. Should any part of the License for any\nreason be judged legally invalid or ineffective under applicable law, such\npartial invalidity or ineffectiveness shall not invalidate the remainder\nof the License, and in such case Affirmer hereby affirms that he or she\nwill not (i) exercise any of his or her remaining Copyright and Related\nRights in the Work or (ii) assert any associated claims and causes of\naction with respect to the Work, in either case contrary to Affirmer's\nexpress Statement of Purpose.\n\n4. Limitations and Disclaimers.\n\n a. No trademark or patent rights held by Affirmer are waived, abandoned,\n    surrendered, licensed or otherwise affected by this document.\n b. Affirmer offers the Work as-is and makes no representations or\n    warranties of any kind concerning the Work, express, implied,\n    statutory or otherwise, including without limitation warranties of\n    title, merchantability, fitness for a particular purpose, non\n    infringement, or the absence of latent or other defects, accuracy, or\n    the present or absence of errors, whether or not discoverable, all to\n    the greatest extent permissible under applicable law.\n c. Affirmer disclaims responsibility for clearing rights of other persons\n    that may apply to the Work or any use thereof, including without\n    limitation any person's Copyright and Related Rights in the Work.\n    Further, Affirmer disclaims responsibility for obtaining any necessary\n    consents, permissions or other rights required for any use of the\n    Work.\n d. Affirmer understands and acknowledges that Creative Commons is not a\n    party to this document and has no duty or obligation with respect to\n    this CC0 or use of the Work.\n",
	    "f1d9dc9f2f4edaaebd9167eb8b79a6da6434313df47921aa4ea8ae278f7739e3": "/**\n * Haketilo demo script.\n *\n * Copyright (C) 2021 Wojtek Kosior\n * Available under the terms of Creative Commons Zero\n * <https://creativecommons.org/publicdomain/zero/1.0/legalcode>\n */\n\n{\n    const banner = document.createElement(\"h2\");\n    \n    banner.textContent = \"Hoooray! Haketilo works :D\";\n    \n    banner.setAttribute(\"style\", `\\\n    margin: 1em; \\\n    border-radius: 1em 0px; \\\n    background-color: #474; \\\n    padding: 10px 20px; \\\n    color: #eee; \\\n    box-shadow: 0 6px 8px 0 rgba(0,0,0,0.24), 0 17px 50px 0 rgba(0,0,0,0.19); \\\n    display: inline-block;\\\n    `);\n    \n    document.body.prepend(banner);\n}\n"
	}
    },
    "repo": ["https://hydrilla.koszko.org/api_v2/"]
}
);

/* Update when changes are made to database schema. Must have 3 elements */
const db_version = [1, 0, 1];

const nr_reductor = ([i, s], num) => [i - 1, s + num * 1024 ** i];
const version_nr = ver => ver.slice(0, 3).reduce(nr_reductor, [2, 0])[1];

let db_version_nr = version_nr(db_version);

const stores = 	[
    ["file",      {keyPath: "sha256"}],
    ["file_uses", {keyPath: "sha256"}],
    ["resource",  {keyPath: "identifier"}],
    ["mapping",   {keyPath: "identifier"}],
    ["setting",   {keyPath: "name"}],
    ["blocking",  {keyPath: "pattern"}],
    ["repo",      {keyPath: "url"}]
];

let db = null;

/* Generate a Promise that resolves when an IndexedDB request succeeds. */
async function wait_request(idb_request)
{
    let resolve, reject;
    const waiter = new Promise((...cbs) => [resolve, reject] = cbs);
    [idb_request.onsuccess, idb_request.onerror] = [resolve, reject];
    return waiter;
}

/* asynchronous wrapper for IDBObjectStore's get() method. */
async function idb_get(transaction, store_name, key)
{
    const req = transaction.objectStore(store_name).get(key);
    return (await wait_request(req)).target.result;
}
this.haketilo_exports["common/indexeddb.js"].idb_get = (idb_get);

/* asynchronous wrapper for IDBObjectStore's put() method. */
async function idb_put(transaction, store_name, object)
{
    return wait_request(transaction.objectStore(store_name).put(object));
}

/* asynchronous wrapper for IDBObjectStore's delete() method. */
async function idb_del(transaction, store_name, key)
{
    return wait_request(transaction.objectStore(store_name).delete(key));
}

async function perform_upgrade(event) {
    if (event.oldVersion > db_version_nr)
	throw "bad db version: " + event.oldVersion;

    const opened_db = event.target.result;
    const transaction = event.target.transaction;

    if (event.oldVersion == 0) {
	for (const [store_name, key_mode] of stores)
	    opened_db.createObjectStore(store_name, key_mode);
    }

    if (event.oldVersion > 0 && event.oldVersion < db_version_nr) {
	const v1_url = "https://hydrilla.koszko.org/api_v1/";
	const v1_entry = await idb_get(transaction, "repo", v1_url);

	if (v1_entry) {
	    const v2_url = "https://hydrilla.koszko.org/api_v2/";

	    await idb_del(transaction, "repo", v1_url);
	    await idb_put(transaction, "repo", {url: v2_url});
	}
    }

    if (event.oldVersion == 0) {
	const ctx = make_context(transaction, initial_data.file);
	await _save_items(initial_data.resource, initial_data.mapping,
			  initial_data.repo || [], ctx);
    } else {
	await new Promise(
	    (...cbs) => [transaction.onsuccess, transaction.onerror] = cbs
	);
    }

    return opened_db;
}

/* Open haketilo database, asynchronously return an IDBDatabase object. */
async function get_db() {
    if (db)
	return db;

    let resolve, reject;
    const waiter = new Promise((...cbs) => [resolve, reject] = cbs);

    const request = indexedDB.open("haketilo", db_version_nr);
    request.onsuccess       = ev => resolve(ev.target.result);
    request.onerror         = ev => reject("db error: " + ev.target.errorCode);
    request.onupgradeneeded = ev => perform_upgrade(ev).then(resolve, reject);

    const opened_db = await waiter;

    if (db)
	opened_db.close();
    else
	db = opened_db;

    return db;
}
this.haketilo_exports["common/indexeddb.js"].get = (get_db);

/* Helper function used by make_context(). */
function reject_discard(context)
{
    broadcast.discard(context.sender);
    broadcast.close(context.sender);
    context.reject();
}

/* Helper function used by make_context(). */
function resolve_flush(context)
{
    broadcast.close(context.sender);
    context.resolve();
}

/* Helper function used by start_items_transaction() and get_db(). */
function make_context(transaction, files)
{
    const sender = broadcast.sender_connection();

    files = files || {};
    let resolve, reject;
    const result = new Promise((...cbs) => [resolve, reject] = cbs);

    const context =
	  {sender, transaction, resolve, reject, result, files, file_uses: {}};

    transaction.oncomplete = () => resolve_flush(context);
    transaction.onerror = () => reject_discard(context);

    return context;
}

/*
 * item_store_names should be an array with either string "mapping", string
 * "resource" or both. files should be an object with an "sha256" property whose
 * values will be yet another object with values being contents of files that
 * are to be possibly saved in this transaction and keys being hexadecimal
 * representations of files' SHA256 sums.
 *
 * Returned is a context object wrapping the transaction and handling the
 * counting of file references in IndexedDB.
 */
async function start_items_transaction(item_store_names, files)
{
    const db = await get_db();
    const scope = [...item_store_names, "file", "file_uses"];
    return make_context(db.transaction(scope, "readwrite"), files);
}
this.haketilo_exports["common/indexeddb.js"].start_items_transaction = (start_items_transaction);

async function incr_file_uses(context, file_ref, by=1)
{
    const sha256 = file_ref.sha256;
    let uses = context.file_uses[sha256];
    if (uses === undefined) {
	uses = await idb_get(context.transaction, "file_uses", sha256);
	if (uses)
	    [uses.new, uses.initial] = [false, uses.uses];
	else
	    uses = {sha256, uses: 0, new: true, initial: 0};

	context.file_uses[sha256] = uses;
    }

    uses.uses = uses.uses + by;
}

const decr_file_uses = (ctx, file_ref) => incr_file_uses(ctx, file_ref, -1);

async function finalize_transaction(context)
{
    for (const uses of Object.values(context.file_uses)) {
	if (uses.uses < 0)
	    console.error("Haketilo: internal error: uses < 0 for file " + uses.sha256);

	const is_new       = uses.new;
	const initial_uses = uses.initial;
	const sha256       = uses.sha256;

	delete uses.new;
	delete uses.initial;

	if (uses.uses < 1) {
	    if (!is_new) {
		idb_del(context.transaction, "file_uses", sha256);
		idb_del(context.transaction, "file",      sha256);
	    }

	    continue;
	}

	if (uses.uses === initial_uses)
	    continue;

	idb_put(context.transaction, "file_uses", uses);

	if (initial_uses > 0)
	    continue;

	const file = context.files.sha256[sha256];
	if (file === undefined) {
	    context.transaction.abort();
	    throw "file not present: " + sha256;
	}

	idb_put(context.transaction, "file", {sha256, contents: file});
    }

    return context.result;
}
this.haketilo_exports["common/indexeddb.js"].finalize_transaction = (finalize_transaction);

/*
 * How a sample data argument to the function below might look like:
 *
 * data = {
 *     resource: {
 *         "resource1": {
 *             "1": {
 *                 // some stuff
 *             },
 *             "1.1": {
 *                 // some stuff
 *             }
 *         },
 *         "resource2": {
 *             "0.4.3": {
 *                 // some stuff
 *             }
 *         },
 *     },
 *     mapping: {
 *         "mapping1": {
 *             "2": {
 *                 // some stuff
 *             }
 *         },
 *         "mapping2": {
 *             "0.1": {
 *                 // some stuff
 *             }
 *         },
 *     },
 *     file: {
 *         sha256: {
 *             "f9444510dc7403e41049deb133f6892aa6a63c05591b2b59e4ee5b234d7bbd99": "console.log(\"hello\");\n",
 *             "b857cd521cc82fff30f0d316deba38b980d66db29a5388eb6004579cf743c6fd": "console.log(\"bye\");"
 *         }
 *     }
 * }
 */
async function save_items(data)
{
    const item_store_names = ["resource", "mapping"];
    if ("repo" in data)
	item_store_names.push("repo");

    const context = await start_items_transaction(item_store_names, data.file);

    return _save_items(data.resource, data.mapping, data.repo || [], context);
}
this.haketilo_exports["common/indexeddb.js"].save_items = (save_items);

async function _save_items(resources, mappings, repos, context)
{
    resources = Object.values(resources || {}).map(entities.get_newest);
    mappings  = Object.values(mappings  || {}).map(entities.get_newest);

    for (const item of resources.concat(mappings))
	await save_item(item, context);

    for (const repo_url of repos) {
	broadcast.prepare(context.sender, "idb_changes_repo", repo_url);
	await idb_put(context.transaction, "repo", {url: repo_url});
    }

    await finalize_transaction(context);
}

/*
 * Save given definition of a resource/mapping to IndexedDB. If the definition
 * (passed as `item`) references files that are not already present in
 * IndexedDB, those files should be provided as values of the `files' object
 * used to create the transaction context.
 *
 * context should be one returned from start_items_transaction() and should be
 * later passed to finalize_transaction() so that files depended on are added to
 * IndexedDB and files that are no longer depended on after this operation are
 * removed from IndexedDB.
 */
async function save_item(item, context)
{
    for (const file_ref of entities.get_files(item))
	await incr_file_uses(context, file_ref);

    broadcast.prepare(context.sender, `idb_changes_${item.type}`,
		      item.identifier);
    await _remove_item(item.type, item.identifier, context, false);
    await idb_put(context.transaction, item.type, item);
}
this.haketilo_exports["common/indexeddb.js"].save_item = (save_item);

/* Helper function used by remove_item() and save_item(). */
async function _remove_item(store_name, identifier, context)
{
    const item = await idb_get(context.transaction, store_name, identifier);
    if (item !== undefined) {
	for (const file_ref of entities.get_files(item))
	    await decr_file_uses(context, file_ref);
    }
}

/*
 * Remove definition of a resource/mapping from IndexedDB.
 *
 * context should be one returned from start_items_transaction() and should be
 * later passed to finalize_transaction() so that files depended on are added to
 * IndexedDB and files that are no longer depended on after this operation are
 * removed from IndexedDB.
 */
async function remove_item(store_name, identifier, context)
{
    broadcast.prepare(context.sender, `idb_changes_${store_name}`, identifier);
    await _remove_item(store_name, identifier, context);
    await idb_del(context.transaction, store_name, identifier);
}

const remove_resource = (id, ctx) => remove_item("resource", id, ctx);
this.haketilo_exports["common/indexeddb.js"].remove_resource = (remove_resource);

const remove_mapping = (id, ctx) => remove_item("mapping",  id, ctx);
this.haketilo_exports["common/indexeddb.js"].remove_mapping = (remove_mapping);

/* Function to retrieve all items from a given store. */
async function get_all(store_name)
{
    const transaction = (await get_db()).transaction([store_name]);
    const all_req = transaction.objectStore(store_name).getAll();

    return (await wait_request(all_req)).target.result;
}
this.haketilo_exports["common/indexeddb.js"].get_all = (get_all);

/*
 * A simplified kind of transaction for modifying stores without special
 * inter-store integrity constraints ("setting", "blocking", "repo").
 */
async function start_simple_transaction(store_name)
{
    const db = await get_db();
    return make_context(db.transaction(store_name, "readwrite"), {});
}

/* Functions to access the "setting" store. */
async function set_setting(name, value)
{
    const context = await start_simple_transaction("setting");
    broadcast.prepare(context.sender, "idb_changes_setting", name);
    await idb_put(context.transaction, "setting", {name, value});
    return finalize_transaction(context);
}
this.haketilo_exports["common/indexeddb.js"].set_setting = (set_setting);

async function get_setting(name)
{
    const transaction = (await get_db()).transaction("setting");
    return ((await idb_get(transaction, "setting", name)) || {}).value;
}
this.haketilo_exports["common/indexeddb.js"].get_setting = (get_setting);

/* Functions to access the "blocking" store. */
async function set_allowed(pattern, allow=true)
{
    const context = await start_simple_transaction("blocking");
    broadcast.prepare(context.sender, "idb_changes_blocking", pattern);
    if (allow === null)
	await idb_del(context.transaction, "blocking", pattern);
    else
	await idb_put(context.transaction, "blocking", {pattern, allow});
    return finalize_transaction(context);
}
this.haketilo_exports["common/indexeddb.js"].set_allowed = (set_allowed);

const set_disallowed = pattern => set_allowed(pattern, false);
this.haketilo_exports["common/indexeddb.js"].set_disallowed = (set_disallowed);

const set_default_allowing = pattern => set_allowed(pattern, null);
this.haketilo_exports["common/indexeddb.js"].set_default_allowing = (set_default_allowing);

async function get_allowing(pattern)
{
    const transaction = (await get_db()).transaction("blocking");
    return ((await idb_get(transaction, "blocking", pattern)) || {}).allow;
}
this.haketilo_exports["common/indexeddb.js"].get_allowing = (get_allowing);

/* Functions to access the "repo" store. */
async function set_repo(url, remove=false)
{
    const context = await start_simple_transaction("repo");
    broadcast.prepare(context.sender, "idb_changes_repo", url);
    if (remove)
	await idb_del(context.transaction, "repo", url);
    else
	await idb_put(context.transaction, "repo", {url});
    return finalize_transaction(context);
}
this.haketilo_exports["common/indexeddb.js"].set_repo = (set_repo);

const del_repo = url => set_repo(url, true);
this.haketilo_exports["common/indexeddb.js"].del_repo = (del_repo);

const get_repos = () => get_all("repo").then(list => list.map(obj => obj.url));
this.haketilo_exports["common/indexeddb.js"].get_repos = (get_repos);

/* Callback used when listening to broadcasts while tracking db changes. */
async function track_change(tracking, key)
{
    const transaction = (await get_db()).transaction([tracking.store_name]);
    const new_val = await idb_get(transaction, tracking.store_name, key);

    tracking.onchange({key, new_val});
}

/*
 * Monitor changes to `store_name` IndexedDB object store.
 *
 * `store_name` should be either "resource", "mapping", "setting", "blocking"
 * or "repo".
 *
 * `onchange` should be a callback that will be called when an item is added,
 * modified or removed from the store. The callback will be passed an object
 * representing the change as its first argument. This object will have the
 * form:
 * {
 *     key: "the identifier of modified resource/mapping or settings key",
 *     new_val: undefined // `undefined` if item removed, item object otherwise
 * }
 *
 * Returns a [tracking, all_current_items] array where `tracking` is an object
 * that can be later passed to untrack() to stop tracking changes and
 * `all_current_items` is an array of items currently present in the object
 * store.
 *
 * It is possible that `onchange` gets spuriously fired even when an item is not
 * actually modified or that it only gets called once after multiple quick
 * changes to an item.
 */
async function start_tracking(store_name, onchange)
{
    const tracking = {store_name, onchange};
    tracking.listener =
	broadcast.listener_connection(msg => track_change(tracking, msg[1]));
    broadcast.subscribe(tracking.listener, `idb_changes_${store_name}`);

    return [tracking, await get_all(store_name)];
}

const track = {};
const trackable = ["resource", "mapping", "setting", "blocking", "repo"];
for (const store_name of trackable)
    track[store_name] = onchange => start_tracking(store_name, onchange);
this.haketilo_exports["common/indexeddb.js"].track = (track);

const untrack = tracking => broadcast.close(tracking.listener);
this.haketilo_exports["common/indexeddb.js"].untrack = (untrack);
}
}).call({
    haketilo_exports: this.haketilo_exports,
    haketilo_this:    this
});
