"use strict";
this.haketilo_exports = this.haketilo_exports || {};
this.haketilo_exports["common/jsonschema.js"] = {};

(function() {
var globalThis = this.haketilo_this
{
/* SPDX-License-Identifier: MIT AND CC0-1.0
 *
 * License text of the original lib/index.js from jsonschema library:
 *
 ***************************************
 *
 * jsonschema is licensed under MIT license.
 *
 * Copyright (C) 2012-2015 Tom de Grunt <tom@degrunt.nl>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 *******************************************************************************
 *
 * License notice for the adaptation to use in Haketilo:
 *
 ***************************************
 *
 * Copyright (C) 2022 Wojtek Kosior <koszko@koszko.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the CC0 1.0 Universal License as published by
 * the Creative Commons Corporation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * CC0 1.0 Universal License for more details.
 */

const Validator = haketilo_exports["common/jsonschema/validator.js"].Validator;
this.haketilo_exports["common/jsonschema.js"].Validator = (Validator);

const ValidatorResult = haketilo_exports["common/jsonschema/helpers.js"].ValidatorResult;
const ValidationError = haketilo_exports["common/jsonschema/helpers.js"].ValidationError;
const ValidatorResultError = haketilo_exports["common/jsonschema/helpers.js"].ValidatorResultError;
const SchemaError = haketilo_exports["common/jsonschema/helpers.js"].SchemaError;

this.haketilo_exports["common/jsonschema.js"].ValidatorResult = (ValidatorResult);
this.haketilo_exports["common/jsonschema.js"].ValidationError = (ValidationError);
this.haketilo_exports["common/jsonschema.js"].ValidatorResultError = (ValidatorResultError);
this.haketilo_exports["common/jsonschema.js"].SchemaError = (SchemaError);

const SchemaScanResult = haketilo_exports["common/jsonschema/scan.js"].SchemaScanResult;
const scan = haketilo_exports["common/jsonschema/scan.js"].scan;

const parse_schema_uri = haketilo_exports["common/entities.js"].parse_schema_uri;

this.haketilo_exports["common/jsonschema.js"].scan = (scan);
this.haketilo_exports["common/jsonschema.js"].SchemaScanResult = (SchemaScanResult);

function validate(instance, schema, options) {
    var v = new Validator();
    return v.validate(instance, schema, options);
};
this.haketilo_exports["common/jsonschema.js"].validate = (validate);

const haketilo_schemas = [
    /* 1.x Hydrilla JSON schema series */
{
    "$schema": "http://json-schema.org/draft-07/schema#",
    "$id": "https://hydrilla.koszko.org/schemas/api_query_result-1.0.1.schema.json",
    "title": "Query result",
    "description": "Object with a list of references to mappings that contain payloads for requested URL",
    "type": "object",
    "required": ["$schema", "mappings"],
    "properties": {
	"$schema": {
	    "description": "Mark this instance as conforming to query result schema 1.x",
	    "type": "string",
	    "pattern": "^https://hydrilla\\.koszko\\.org/schemas/api_query_result-1\\.(([1-9][0-9]*|0)\\.)*schema\\.json$"
	},
	"mappings": {
	    "description": "References to mappings using at least one pattern that matches the requested URL",
	    "type": "array",
	    "items": {
		"$ref": "./common_definitions-1.0.1.schema.json#/definitions/item_ref"
	    }
	},
	"generated_by": {
	    "$ref": "./common_definitions-1.0.1.schema.json#/definitions/generated_by"
	}
    }
}
    ,
{
    "$schema": "http://json-schema.org/draft-07/schema#",
    "$id": "https://hydrilla.koszko.org/schemas/api_mapping_description-1.0.1.schema.json",
    "title": "Mapping description",
    "description": "Definition of a Hydrilla mapping, as served through HTTP API",
    "allOf": [{
	"$ref": "./common_definitions-1.0.1.schema.json#/definitions/mapping_definition_base"
    }, {
	"$ref": "./common_definitions-1.0.1.schema.json#/definitions/item_definition"
    }, {
	"type": "object",
	"required": ["$schema"],
	"properties": {
	    "$schema": {
		"description": "Mark this instance as conforming to mapping description schema 1.x",
		"type": "string",
		"pattern": "^https://hydrilla\\.koszko\\.org/schemas/api_mapping_description-1\\.(([1-9][0-9]*|0)\\.)*schema\\.json$"
	    }
	}
    }]
}
    ,
{
    "$schema": "http://json-schema.org/draft-07/schema#",
    "$id": "https://hydrilla.koszko.org/schemas/api_resource_description-1.0.1.schema.json",
    "title": "Resource description",
    "description": "Definition of a Hydrilla resource, as served through HTTP API",
    "allOf": [{
	"$ref": "./common_definitions-1.0.1.schema.json#/definitions/resource_definition_base"
    }, {
	"$ref": "./common_definitions-1.0.1.schema.json#/definitions/item_definition"
    }, {
	"type": "object",
	"required": ["$schema"],
	"properties": {
	    "$schema": {
		"description": "Mark this instance as conforming to resource description schema 1.x",
		"type": "string",
		"pattern": "^https://hydrilla\\.koszko\\.org/schemas/api_resource_description-1\\.(([1-9][0-9]*|0)\\.)*schema\\.json$"
	    },
	    "scripts": {
		"description": "Which files are resource's scripts and need to be installed",
		"$ref": "./common_definitions-1.0.1.schema.json#/definitions/file_ref_list_sha256",
		"default": []
	    }
	}
    }]
}
    ,
{
    "$schema": "http://json-schema.org/draft-07/schema#",
    "$id": "https://hydrilla.koszko.org/schemas/common_definitions-1.0.1.schema.json",
    "title": "Common definitions",
    "description": "Definitions used by other Hydrilla schemas",
    "definitions": {
	"version": {
	    "description": "Version expressed as an array of integers",
	    "type": "array",
	    "minItems": 1,
	    "items": {
		"type": "integer",
		"minimum": 0
	    },
	    "contains": {
		"type": "integer",
		"minimum": 1
	    },
	    "minItems": 1
	},
	"source_name": {
	    "description": "Unique identifier of this source package",
	    "type": "string",
	    "pattern": "^[-0-9a-z.]+$"
	},
	"comment": {
	    "description": "An optional comment",
	    "type": "string"
	},
        "file_ref_list": {
	    "description": "List of simple file references",
	    "type": "array",
	    "items": {
		"type": "object",
		"required": ["file"],
		"properties": {
		    "file": {
			"description": "Filename relative to source package main directory; separator is '/'",
			"type": "string",
			"pattern": "^[^/]"
		    }
		}
	    }
        },
	"sha256": {
	    "description": "An SHA256 sum, in hexadecimal",
	    "type": "string",
	    "pattern": "^[0-9a-f]{64}$"
	},
        "file_ref_list_sha256": {
	    "description": "List of file references with files' SHA256 sums included",
	    "allOf": [{
		"$ref": "#/definitions/file_ref_list"
	    }, {
		"type": "array",
		"items": {
		    "type": "object",
		    "required": ["sha256"],
		    "properties": {
			"sha256": {
			    "$ref": "#/definitions/sha256"
			}
		    }
		}
	    }]
        },
	"item_identifier": {
	    "description": "Identifier of an item (shared with other versions of the item, otherwise unique)",
	    "type": "string",
	    "pattern": "^[-0-9a-z]+$"
	},
	"item_dep_specifier": {
	    "description": "Simple reference to an item as a dependency",
	    "type": "object",
	    "required": ["identifier"],
	    "properties": {
		"identifier": {
		    "$ref": "#/definitions/item_identifier"
		}
	    }
	},
	"item_ref": {
	    "description": "An object containing a subset of fields from full item definition",
	    "type": "object",
	    "required": ["identifier", "long_name", "version"],
	    "properties": {
		"identifier": {
		    "$ref": "#/definitions/item_identifier"
		},
		"long_name": {
		    "description": "User-friendly alternative to the identifier",
		    "type": "string"
		},
		"version": {
		    "$ref": "#/definitions/version"
		}
	    }
	},
	"typed_item_ref": {
	    "description": "An object containing a subset of fields from full item definition, including type",
	    "allOf": [{
		"$ref": "#/definitions/item_ref"
	    }, {
		"type": "object",
		"required": ["type"],
		"properties": {
		    "type": {
			"description": "What kind of item is it (resource or mapping)",
			"enum": ["resource", "mapping"]
		    }
		}
	    }]
	},
	"item_definition_base": {
	    "description": "Definition of a resource/mapping (fields common to source definitions and built definitions)",
	    "allOf": [{
		"$ref": "#/definitions/typed_item_ref"
	    }, {
		"type": "object",
		"required": ["description"],
		"properties": {
		    "uuid": {
			"description": "UUIDv4 of this item (shared with other versions of this item, otherwise unique)",
			"type": "string",
			"pattern": "^[0-9a-f]{8}-[0-9a-f]{4}-4[0-9a-f]{3}-[89ab][0-9a-f]{3}-[0-9a-f]{12}$"
		    },
		    "description": {
			"description": "Item's description",
			"type": "string"
		    },
		    "comment": {
			"$ref": "#/definitions/comment"
		    }
		}
	    }]
	},
	"resource_definition_base": {
	    "description": "Definition of a resource (fields common to source definitions and built definitions)",
	    "allOf": [{
		"$ref": "#/definitions/item_definition_base"
	    }, {
		"type": "object",
		"required": ["type", "revision"],
		"properties": {
		    "type": {
			"description": "Identify this item as a resource",
			"const": "resource"
		    },
		    "revision": {
			"description": "Which revision of a packaging of given version of an upstream resource is this",
			"type": "integer",
			"minimum": 1
		    },
		    "scripts": {
			"description": "What scripts are included in the resource",
			"$ref": "#/definitions/file_ref_list",
			"default": []
		    },
		    "dependencies": {
			"description": "Which other resources this resource depends on",
			"type": "array",
			"items": {
			    "$ref": "#/definitions/item_dep_specifier"
			},
			"default": []
		    }
		}
	    }]
	},
	"mapping_definition_base": {
	    "description": "Definition of a mapping (fields common to source definitions and built definitions)",
	    "allOf": [{
		"$ref": "#/definitions/item_definition_base"
	    }, {
		"type": "object",
		"required": ["type"],
		"properties": {
		    "type": {
			"description": "Identify this item as a mapping",
			"const": "mapping"
		    },
		    "payloads": {
			"description": "Which payloads are to be applied to which URLs",
			"additionalProperties": {
			    "$ref": "#/definitions/item_dep_specifier"
			},
			"default": {},
			"examples": [{
			    "https://hydrillabugs.koszko.org/***": {
				"identifier": "helloapple"
			    },
			    "https://*.koszko.org/***": {
				"identifier": "hello-potato"
			    }
			}]
		    }
		}
	    }]
	},
	"generated_by": {
	    "description": "Describe what software generated this instance",
	    "type": "object",
	    "required": ["name"],
	    "properties": {
		"name": {
		    "type": "string",
		    "description": "Instance generator software name, without version"
		},
		"version": {
		    "type": "string",
		    "description": "Instance generator software version, in arbitrary format"
		}
	    }
	},
	"item_definition": {
	    "description": "Definition of a resource/mapping (fields specific to built definitions)",
	    "type": "object",
	    "required": ["source_name", "source_copyright"],
	    "properties": {
		"source_name": {
		    "$ref": "#/definitions/source_name"
		},
		"source_copyright": {
		    "description": "Which files indicate license terms of the source package and should be installed",
		    "$ref": "#/definitions/file_ref_list_sha256"
		},
		"generated_by": {
		    "$ref": "#/definitions/generated_by"
		}
	    }
	}
    }
}
    ,
    /* 2.x Hydrilla JSON schema series */
{
    "$schema": "http://json-schema.org/draft-07/schema#",
    "$id": "https://hydrilla.koszko.org/schemas/api_query_result-2.schema.json",
    "title": "Query result",
    "description": "Object with a list of references to mappings that contain payloads for requested URL",
    "type": "object",
    "required": ["$schema", "mappings"],
    "properties": {
	"$schema": {
	    "description": "Mark this instance as conforming to query result schema 2.x",
	    "type": "string",
	    "pattern": "^https://hydrilla\\.koszko\\.org/schemas/api_query_result-2\\.(([1-9][0-9]*|0)\\.)*schema\\.json$"
	},
	"mappings": {
	    "description": "References to mappings using at least one pattern that matches the requested URL",
	    "type": "array",
	    "items": {
		"$ref": "./common_definitions-2.schema.json#/definitions/item_ref"
	    }
	},
	"generated_by": {
	    "$ref": "./common_definitions-2.schema.json#/definitions/generated_by"
	}
    }
}
    ,
{
    "$schema": "http://json-schema.org/draft-07/schema#",
    "$id": "https://hydrilla.koszko.org/schemas/api_mapping_description-2.schema.json",
    "title": "Mapping description",
    "description": "Definition of a Hydrilla mapping, as served through HTTP API",
    "allOf": [{
	"$ref": "./common_definitions-2.schema.json#/definitions/mapping_definition_base"
    }, {
	"$ref": "./common_definitions-2.schema.json#/definitions/item_definition"
    }, {
	"type": "object",
	"required": ["$schema", "type"],
	"properties": {
	    "$schema": {
		"description": "Mark this instance as conforming to mapping description schema 2.x",
		"type": "string",
		"pattern": "^https://hydrilla\\.koszko\\.org/schemas/api_mapping_description-2\\.(([1-9][0-9]*|0)\\.)*schema\\.json$"
	    },
	    "type": {
		"description": "Identify this item as a mapping",
		"const": "mapping"
	    }
	}
    }]
}
    ,
{
    "$schema": "http://json-schema.org/draft-07/schema#",
    "$id": "https://hydrilla.koszko.org/schemas/api_resource_description-2.schema.json",
    "title": "Resource description",
    "description": "Definition of a Hydrilla resource, as served through HTTP API",
    "allOf": [{
	"$ref": "./common_definitions-2.schema.json#/definitions/resource_definition_base"
    }, {
	"$ref": "./common_definitions-2.schema.json#/definitions/item_definition"
    }, {
	"type": "object",
	"required": ["$schema", "type"],
	"properties": {
	    "$schema": {
		"description": "Mark this instance as conforming to resource description schema 2.x",
		"type": "string",
		"pattern": "^https://hydrilla\\.koszko\\.org/schemas/api_resource_description-2\\.(([1-9][0-9]*|0)\\.)*schema\\.json$"
	    },
	    "type": {
		"description": "Identify this item as a resource",
		"const": "resource"
	    },
	    "scripts": {
		"description": "Which files are resource's scripts and need to be installed",
		"$ref": "./common_definitions-2.schema.json#/definitions/file_ref_list_sha256",
		"default": []
	    },
	    "dependencies": {
		"description": "Which other resources this resource depends on",
		"$ref": "./common_definitions-2.schema.json#/definitions/item_dep_specifier_array",
		"default": []
	    }
	}
    }]
}
    ,
{
    "$schema": "http://json-schema.org/draft-07/schema#",
    "$id": "https://hydrilla.koszko.org/schemas/common_definitions-2.schema.json",
    "title": "Common definitions",
    "description": "Definitions used by other Hydrilla schemas",
    "definitions": {
	"version": {
	    "description": "Version expressed as an array of integers",
	    "type": "array",
	    "minItems": 1,
	    "items": {
		"type": "integer",
		"minimum": 0
	    },
	    "contains": {
		"type": "integer",
		"minimum": 1
	    },
	    "minItems": 1
	},
	"source_name": {
	    "description": "Unique identifier of this source package",
	    "type": "string",
	    "pattern": "^[-0-9a-z.]+$"
	},
	"comment": {
	    "description": "An optional comment",
	    "type": "string"
	},
        "file_ref_list": {
	    "description": "List of simple file references",
	    "type": "array",
	    "items": {
		"type": "object",
		"required": ["file"],
		"properties": {
		    "file": {
			"description": "Filename relative to source package main directory; separator is '/'",
			"type": "string",
			"pattern": "^[^/]"
		    }
		}
	    }
        },
	"sha256": {
	    "description": "An SHA256 sum, in hexadecimal",
	    "type": "string",
	    "pattern": "^[0-9a-f]{64}$"
	},
        "file_ref_list_sha256": {
	    "description": "List of file references with files' SHA256 sums included",
	    "allOf": [{
		"$ref": "#/definitions/file_ref_list"
	    }, {
		"type": "array",
		"items": {
		    "type": "object",
		    "required": ["sha256"],
		    "properties": {
			"sha256": {
			    "$ref": "#/definitions/sha256"
			}
		    }
		}
	    }]
        },
	"item_identifier": {
	    "description": "Identifier of an item (shared with other versions of the item, otherwise unique)",
	    "type": "string",
	    "pattern": "^[-0-9a-z]+$"
	},
	"item_dep_specifier": {
	    "description": "Simple reference to an item as a dependency",
	    "type": "object",
	    "required": ["identifier"],
	    "properties": {
		"identifier": {
		    "$ref": "#/definitions/item_identifier"
		}
	    }
	},
	"item_dep_specifier_array": {
	    "description": "Array of references to items as dependencies",
	    "type": "array",
	    "items": {
		"$ref": "#/definitions/item_dep_specifier"
	    }
	},
	"item_ref": {
	    "description": "An object containing a subset of fields from full item definition",
	    "type": "object",
	    "required": ["identifier", "long_name", "version"],
	    "properties": {
		"identifier": {
		    "$ref": "#/definitions/item_identifier"
		},
		"long_name": {
		    "description": "User-friendly alternative to the identifier",
		    "type": "string"
		},
		"version": {
		    "$ref": "#/definitions/version"
		}
	    }
	},
	"typed_item_ref": {
	    "description": "An object containing a subset of fields from full item definition, including type",
	    "allOf": [{
		"$ref": "#/definitions/item_ref"
	    }, {
		"type": "object",
		"required": ["type"],
		"properties": {
		    "type": {
			"description": "What kind of item is it (resource or mapping)",
			"enum": ["resource", "mapping"]
		    }
		}
	    }]
	},
	"item_definition_base": {
	    "description": "Definition of a resource/mapping (fields common to source definitions and built definitions)",
	    "allOf": [{
		"$ref": "#/definitions/item_ref"
	    }, {
		"type": "object",
		"required": ["description"],
		"properties": {
		    "uuid": {
			"description": "UUIDv4 of this item (shared with other versions of this item, otherwise unique)",
			"type": "string",
			"pattern": "^[0-9a-f]{8}-[0-9a-f]{4}-4[0-9a-f]{3}-[89ab][0-9a-f]{3}-[0-9a-f]{12}$"
		    },
		    "description": {
			"description": "Item's description",
			"type": "string"
		    },
		    "min_haketilo_version": {
			"description": "Specify that this item should not be used with Haketilo versions older than specified here",
			"$ref": "#/definitions/version",
			"default": [1]
		    },
		    "max_haketilo_version": {
			"description": "Specify that this item should not be used with Haketilo versions newer than specified here",
			"$ref": "#/definitions/version",
			"default": [65536]
		    },
		    "permissions": {
			"description": "What privileges should be granted on pages where this resource/mapping is used",
			"type": "object",
			"properties": {
			    "cors_bypass": {
				"description": "Specify if a page should be allowed to perform cross-origin requests",
				"type": "boolean",
				"default": false
			    },
			    "eval": {
				"description": "Specify if scripts added to the page should be allowed to use eval() (and related mechanisms)",
				"type": "boolean",
				"default": false
			    }
			}
		    },
		    "required_mappings": {
			"description": "Which mappings this item requires to be enabled",
			"$ref": "#/definitions/item_dep_specifier_array",
			"default": []
		    },
		    "comment": {
			"$ref": "#/definitions/comment"
		    }
		}
	    }]
	},
	"resource_definition_base": {
	    "description": "Definition of a resource (fields common to source definitions and built definitions)",
	    "allOf": [{
		"$ref": "#/definitions/item_definition_base"
	    }, {
		"type": "object",
		"required": ["type", "revision"],
		"properties": {
		    "revision": {
			"description": "Which revision of a packaging of given version of an upstream resource is this",
			"type": "integer",
			"minimum": 1
		    },
		    "scripts": {
			"description": "What scripts are included in the resource",
			"$ref": "#/definitions/file_ref_list",
			"default": []
		    }
		}
	    }]
	},
	"mapping_definition_base": {
	    "description": "Definition of a mapping (fields common to source definitions and built definitions)",
	    "allOf": [{
		"$ref": "#/definitions/item_definition_base"
	    }, {
		"type": "object",
		"properties": {
		    "payloads": {
			"description": "Which payloads are to be applied to which URLs",
			"additionalProperties": {
			    "$ref": "#/definitions/item_dep_specifier"
			},
			"default": {},
			"examples": [{
			    "https://hydrillabugs.koszko.org/***": {
				"identifier": "helloapple"
			    },
			    "https://*.koszko.org/***": {
				"identifier": "hello-potato"
			    }
			}]
		    }
		}
	    }]
	},
	"generated_by": {
	    "description": "Describe what software generated this instance",
	    "type": "object",
	    "required": ["name"],
	    "properties": {
		"name": {
		    "type": "string",
		    "description": "Instance generator software name, without version"
		},
		"version": {
		    "type": "string",
		    "description": "Instance generator software version, in arbitrary format"
		}
	    }
	},
	"item_definition": {
	    "description": "Definition of a resource/mapping (fields specific to built definitions)",
	    "type": "object",
	    "required": ["source_name", "source_copyright"],
	    "properties": {
		"source_name": {
		    "$ref": "#/definitions/source_name"
		},
		"source_copyright": {
		    "description": "Which files indicate license terms of the source package and should be installed",
		    "$ref": "#/definitions/file_ref_list_sha256"
		},
		"generated_by": {
		    "$ref": "#/definitions/generated_by"
		}
	    }
	}
    }
}
].reduce((ac, s) => Object.assign(ac, {[s.$id]: s}), {});

for (const [$id, schema] of [...Object.entries(haketilo_schemas)]) {
    const parsed = parse_schema_uri($id);
    const schema_name = `${parsed.name_base}-${parsed.major}.schema.json`;
    haketilo_schemas[schema_name] = schema;
}

this.haketilo_exports["common/jsonschema.js"].haketilo_schemas = (haketilo_schemas);

const haketilo_validator = new Validator();
Object.values(haketilo_schemas)
    .forEach(s => haketilo_validator.addSchema(s, s.$id));
this.haketilo_exports["common/jsonschema.js"].haketilo_validator = (haketilo_validator);
}
}).call({
    haketilo_exports: this.haketilo_exports,
    haketilo_this:    this
});
