"use strict";
this.haketilo_exports = this.haketilo_exports || {};
this.haketilo_exports["content/content.js"] = {};

(function() {
var globalThis = this.haketilo_this
{
/**
 * This file is part of Haketilo.
 *
 * Function: Content scripts - main script.
 *
 * Copyright (C) 2022 Wojtek Kosior
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * As additional permission under GNU GPL version 3 section 7, you
 * may distribute forms of that code without the copy of the GNU
 * GPL normally required by section 4, provided you include this
 * license notice and, in case of non-source distribution, a URL
 * through which recipients can access the Corresponding Source.
 * If you modify file(s) with this exception, you may extend this
 * exception to your version of the file(s), but you are not
 * obligated to do so. If you do not wish to do so, delete this
 * exception statement from your version.
 *
 * As a special exception to the GPL, any HTML file which merely
 * makes function calls to this code, and for that purpose
 * includes it by reference shall be deemed a separate work for
 * copyright law purposes. If you modify this code, you may extend
 * this exception to your version of the code, but you are not
 * obligated to do so. If you do not wish to do so, delete this
 * exception statement from your version.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 * I, Wojtek Kosior, thereby promise not to sue for violation of this file's
 * license. Although I request that you do not make use of this code in a
 * proprietary program, I am not going to enforce this in court.
 */

const repo_query_cacher = haketilo_exports["content/repo_query_cacher.js"];
const haketilo_apis = haketilo_exports["content/haketilo_apis.js"];

const browser = haketilo_exports["common/browser.js"].browser;
const is_privileged_url = haketilo_exports["common/misc.js"].is_privileged_url;
const decide_policy = haketilo_exports["common/policy.js"].decide_policy;
const fallback_policy = haketilo_exports["common/policy.js"].fallback_policy;
const enforce_blocking = haketilo_exports["content/policy_enforcing.js"].enforce_blocking;


let already_run = false, resolve_page_info,
    page_info_prom = new Promise(cb => resolve_page_info = cb);

function on_page_info_request([type], sender, respond_cb) {
    if (type !== "page_info")
	return;

    page_info_prom.then(respond_cb);

    return true;
}

globalThis.haketilo_content_script_main = async function() {
    if (already_run)
        return;

    already_run = true;

    if (is_privileged_url(document.URL))
	return;

    browser.runtime.onMessage.addListener(on_page_info_request);
    repo_query_cacher.start();

    try {
	var policy = decide_policy(globalThis.haketilo_pattern_tree,
				   document.URL,
				   globalThis.haketilo_default_allow,
				   globalThis.haketilo_secret);
    } catch(e) {
	console.error("Haketilo:", e);
	var policy = fallback_policy();
    }

    const page_info = Object.assign({url: document.URL}, policy);
    ["csp", "nonce"].forEach(prop => delete page_info[prop]);

    if ("payload" in policy) {
	const msg = ["indexeddb_files", policy.payload.identifier];
	var scripts_prom = browser.runtime.sendMessage(msg);
    }

    await enforce_blocking(policy);

    if ("payload" in policy) {
	const script_response = await scripts_prom;

	if ("error" in script_response) {
	    resolve_page_info(Object.assign(page_info, script_response));
	    return;
	} else {
	    haketilo_apis.start();

	    const version = browser.runtime.getManifest().version;
	    const scripts = [
		`window.haketilo_version = ${JSON.stringify(version)};`,
		...script_response.files
	    ];
	    for (const script_contents of scripts) {
		const html_ns = "http://www.w3.org/1999/xhtml";
		const script = document.createElementNS(html_ns, "script");
		const load_prom = new Promise(
		    (...cbs) => [script.onload, script.onerror] = cbs
		);

		const blobby_opts = {type: "text/javascript;charset=UTF-8"};
		const blobby = new Blob([script_contents], blobby_opts);
		script.src = URL.createObjectURL(blobby);
		script.setAttribute("nonce", policy.nonce);

		document.documentElement.append(script);
		await load_prom;
		script.remove();
	    }
	}
    }

    resolve_page_info(page_info);
}

function main() {
    if (globalThis.haketilo_pattern_tree !== undefined)
	globalThis.haketilo_content_script_main();
}

main();
}
}).call({
    haketilo_exports: this.haketilo_exports,
    haketilo_this:    this
});
